<form class="row">
    {% csrf_token %}
    <div class="col">
        {% block filter_content %}{% endblock %}
        <input type="text" hidden value="{% block filter_key %}{% endblock %}" name="filter_key">
        <input type="text" hidden value="{% block filter_type %}{% endblock %}" name="filter_type">
    </div>
    <div class="col-auto {% block button_class %} align-self-end {% endblock %}">
        <div class="row">
            {% block submit_button %}{% endblock %}
        </div>
    </div>
</form>

{% block clear_button %}{% endblock %}
